==> openhbci-0.9.18.tar.gz: openhbci-0.9.18/src/plugins/keyfile/keyfile.cpp <==
/***************************************************************************
  $RCSfile: keyfile.cpp,v $
  -------------------
  cvs         : $Id: keyfile.cpp,v 1.11 2005/09/17 16:28:05 cstim Exp $
  begin       : Mon Apr 14 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef __declspec
# if BUILDING_DLL
#  define DLLIMPORT __declspec (dllexport)
# else /* Not BUILDING_DLL */
#  define DLLIMPORT __declspec (dllimport)
# endif /* Not BUILDING_DLL */
#else
# define DLLIMPORT
#endif


#include "openhbci.h"
#include "keyfile.h"
#include "mediumkeyfile.h"
#include "hbcistring.h"
#include <string.h>


namespace HBCI {



KeyFilePlugin::KeyFilePlugin(const API *api)
    :MediumPlugin()
,_api(api)
{

}


KeyFilePlugin::~KeyFilePlugin(){
}


Pointer<Medium> KeyFilePlugin::mediumFactory(const string &name){
  Pointer<Medium> m;

  m=new MediumKeyfile(_api, name);
  return m;
}



Error KeyFilePlugin::mediumCheck(string &name) const {
  string tmp;
  int size;
  Error err;
  Pointer<File> f;

  if (name.empty()) {
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_ABORT,
		 "Empty name.");
  }

  f=new File(name);

  // check whether the file exists
  err=f.ref().accessFile(FILE_ACCESS_EXIST);
  if (!err.isOk()) {
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_ABORT,
		 "File does not exist.");
  }

  // check whether the file is read- and writeable
  err=f.ref().accessFile(FILE_ACCESS_RW);
  if (!err.isOk()) {
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_ABORT,
		 "File exists but I have no rights on it.");
  }

  // ok, now open the file
  err=f.ref().openFile(FILE_AM_OPEN_EXISTING);
  if (!err.isOk()) {
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_ABORT,
		 "File exists, I have all rights but I could not open it.");
  }

  // read tag type and size
  err=f.ref().readData(tmp,3);
  f.ref().closeFile();
  if (!err.isOk()) {
    return Error("KeyfilePlugin::mediumCheck",
		 err);
  }

  if (tmp.length()!=3)
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_BAD_MEDIUM,
		 ERROR_ADVISE_ABORT,
		 "This seems not to be a key file.");
  unsigned typetlv = String::typeTLV(tmp);
  if (typetlv != MEDIUMKEYFILE_TAG_CRYPT_OHBCI_OLD &&
      typetlv != MEDIUMKEYFILE_TAG_CRYPT_OHBCI) {
    if (typetlv == MEDIUMKEYFILE_TAG_CRYPT_AQBANKING3)
      std::cerr << "The key file is of the newer AQBANKING3 type which cannot be read by this openhbci1 library. Sorry." << std::endl;
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_BAD_MEDIUM,
		 ERROR_ADVISE_ABORT,
		 "This seems not to be a key file (bad type).");
  }
  size=String::sizeTLV(tmp);
  if (size % 8)
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_BAD_MEDIUM,
		 ERROR_ADVISE_ABORT,
		 "This seems not to be a key file (bad size).");
  return Error();
}





Pointer<Medium> KeyFilePlugin::createNewMedium(bool readonly,
					       int country,
					       const string &bankId,
					       const string &userId,
					       const string &name){
  Pointer<Medium> medium;
  Error err;

  // name empty ?
  if (name.empty())
    throw Error("KeyFilePlugin::createNewMedium()",
		ERROR_LEVEL_NORMAL,
		0,
		ERROR_ADVISE_DONTKNOW,
		"need name"
		"for RDH mode the name contains the path for the keyfile");

  // check if file already exists
  File f(name);
  if (!f.accessFile(FILE_ACCESS_EXIST).isOk()) {
    // File does not exist

    // create medium; Create new MediumRDH object
    MediumKeyfile *mediumtmp = new MediumKeyfile(_api,name);
    // Store it in the upcast version
    medium = mediumtmp;
    // For convenience, use a reference in the following
    MediumKeyfile &mediumrdh = *mediumtmp;

    string pin;

    err=_api->authentificator().ref().getSecret(0,
						name,
						pin,
						true);
    if (!err.isOk())
      throw Error("KeyFilePlugin::createNewMedium",err);

    // create context
    err=mediumrdh.createMedium(country,
			       bankId,
			       userId,
			       pin);
    if (!err.isOk())
      throw Error("KeyFilePlugin::createNewMedium()", err);

    // mount medium without customer
    err=mediumrdh.mountMedium(pin);
    if (!err.isOk())
      throw Error("KeyFilePlugin::createNewMedium()", err);

    // select context
    err=mediumrdh.selectContext(country, bankId, userId);
    if (!err.isOk())
      throw Error("KeyFilePlugin::createNewMedium()", err);

    // create keys
    err=mediumrdh.createUserKeys();
    if (!err.isOk())
      throw Error("KeyFilePlugin::createNewMedium()", err);

    // unmount medium
    err=mediumrdh.unmountMedium(pin);
    if (!err.isOk())
      throw Error("KeyFilePlugin::createNewMedium()", err);
  } // if not exists
  else
    // File exists, so: create medium
    medium=new MediumKeyfile(_api,name);

  // finished

  return medium;
}


Pointer<Medium> KeyFilePlugin::mediumFromConfig(SimpleConfig &cfg,
						cfgPtr group){
  string typ;
  string name;
  Pointer<Medium> m;

  typ=cfg.getVariable("mediumtype","",group);
  if (strcasecmp(typ.c_str(), mediumTypeName().c_str())!=0) {
    throw Error("KeyFilePlugin::mediumFromConfig",
		ERROR_LEVEL_NORMAL,
		HBCI_ERROR_CODE_INVALID,
		ERROR_ADVISE_DONTKNOW,
		"bad medium type");
  }
  name=cfg.getVariable("mediumname",
		       cfg.getVariable("keyfile","",group),
		       group);
  m=new MediumKeyfile(_api, name);
  return m;
}


Error KeyFilePlugin::mediumToConfig(Pointer<Medium> m,
				    SimpleConfig &cfg,
				    cfgPtr group){
  Pointer<MediumKeyfile> mkf;

  try {
    mkf=m.cast<MediumKeyfile>();
  }
  catch (Error xerr) {
    return Error("KeyFilePlugin::mediumToConfig",xerr);
  }
  cfg.setVariable("mediumtype",m.ref().mediumTypeName(),group);
  cfg.setVariable("mediumname",mkf.ref().mediumName(),group);

  return Error();
}


Error _checkVersion(const API *api) {
  int vmajor, vminor, vpl, vbuild;

  /* do some version checking
   * Please note that the constants used below are defined upon
   * compilation of this plugin. If you compile this plugin using
   * OpenHBCI 0.9.10 and try to use it on a system where OpenHBCI 0.9.9
   * is installed, this module will refuse to work. This plugin only
   * works with versions of OpenHBCI which are the same or newer than
   * the version used to compile this plugin.
   * If the major version differs then this module believes that the
   * version of OpenHBCI used is totally incompatible with the version
   * used to build this plugin.
   */
  api->libraryVersion(vmajor, vminor, vpl, vbuild);
  if (vmajor!=OPENHBCI_VERSION_MAJOR) {
    /* this is the major compatibility check. The major version is
     * incremented if a newer version is incompatible with older ones */
    fprintf(stderr,
	    " Different major versions, "
	    "please recompile RDHFile plugin.\n");
    return HBCI::Error("Keyfile Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "Major version does not match");
  }
  if (!(
	((vminor==OPENHBCI_VERSION_MINOR)&&(vpl>OPENHBCI_VERSION_PATCHLEVEL))
	||
	((vminor==OPENHBCI_VERSION_MINOR) &&
	 (vpl==OPENHBCI_VERSION_PATCHLEVEL) &&
	 (vbuild>=OPENHBCI_VERSION_BUILD))
       )
     ) {
    fprintf(stderr,
	    "This plugin needs OpenHBCI v%d.%d.%d.%d or newer.\n",
	    OPENHBCI_VERSION_MAJOR,
	    OPENHBCI_VERSION_MINOR,
	    OPENHBCI_VERSION_PATCHLEVEL,
            OPENHBCI_VERSION_BUILD);
    return HBCI::Error("RDHFile Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "need OpenHBCI v"OPENHBCI_VERSION_FULL_STRING
		       " or newer");
  }

  return Error();
}



}; // namespace


extern "C" {
  // The version of openhbci's plugin interface system (i.e. the
  // convention that this plugin has to define a registerYourSelf()
  // function.)
  int rdhfile_openhbci_plugin_interface_version = OPENHBCI_PLUGIN_INTERFACE_VERSION;

  // Version numbers of this plugin
  int rdhfile_openhbci_module_current=KEYFILE_MODULE_CURRENT;
  int rdhfile_openhbci_module_revision=KEYFILE_MODULE_REVISION;
  int rdhfile_openhbci_module_age=KEYFILE_MODULE_AGE;


  // The function that registers this plugin at the HBCI::API. This
  // function will be looked up by dlsym() and called from the plugin
  // interface system.
  HBCI::Error rdhfile_registerYourSelf(HBCI::API *api) {
    HBCI::Pointer<HBCI::MediumPlugin> mp;
    HBCI::Error err;

    err=_checkVersion(api);
    if (!err.isOk())
      return HBCI::Error("Keyfile Plugin",err);

    mp=new HBCI::KeyFilePlugin(api);
    return api->registerMediumPlugin(mp);
  }

  /*
   * This function simply creates a medium plugin without registering it
   */
  HBCI::Pointer<HBCI::MediumPlugin> rdhfile_createPlugin(const HBCI::API *api){
    HBCI::Pointer<HBCI::MediumPlugin> mp;
    HBCI::Error err;

    err=_checkVersion(api);
    if (!err.isOk())
      throw HBCI::Error("Keyfile Plugin",err);

    mp=new HBCI::KeyFilePlugin(api);
    mp.setDescription("KeyFilePlugin");
    return mp;
  }


}; // extern "C"

==> openhbci2-1.9.3.tar.gz: openhbci2-1.9.3/src/plugins/keyfile/keyfile.cpp <==
/***************************************************************************
  $RCSfile: keyfile.cpp,v $
  -------------------
  cvs         : $Id: keyfile.cpp,v 1.6 2004/05/19 18:59:56 aquamaniac Exp $
  begin       : Mon Apr 14 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef __declspec
# if BUILDING_DLL
#  define DLLIMPORT __declspec (dllexport)
# else /* Not BUILDING_DLL */
#  define DLLIMPORT __declspec (dllimport)
# endif /* Not BUILDING_DLL */
#else
# define DLLIMPORT
#endif

#include <openhbci2.h>
#include "keyfile.h"
#include "mediumkeyfile.h"
#include "hbcistring.h"
#include <string.h>
#include <openhbci2/pluginmanager.h>

#include <gwenhywfar/text.h>
#include <gwenhywfar/debug.h>

#include <unistd.h>
#include <errno.h>


namespace HBCI {



KeyFilePlugin::KeyFilePlugin(Hbci *hbci)
    :MediumPlugin(hbci)
{

}


KeyFilePlugin::~KeyFilePlugin(){
}


Pointer<Medium> KeyFilePlugin::mediumFactory(const string &name){
  Pointer<Medium> m;

  m=new MediumKeyfile(hbci(), name);
  return m;
}



Error KeyFilePlugin::mediumCheck(string &name) const {
  int size;
  Error err;
  FILE *f;
  char buffer[16];
  int rv;
  string tmp;
  int tlvType;

  if (name.empty()) {
    DBG_INFO(0, "Empty name");
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_ABORT,
		 "Empty name.");
  }

  // check whether the file exists
  DBG_DEBUG(0, "Checking whether file \"%s\" exists", name.c_str());
  if (access(name.c_str(), F_OK)) {
    DBG_INFO(0, "File does not exist");
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_ABORT,
		 "File does not exist.");
  }

  // check whether the file is read- and writeable
  DBG_DEBUG(0, "Checking whether file \"%s\" is read/writeable",
	   name.c_str());
  if (access(name.c_str(), R_OK | W_OK)) {
    DBG_INFO(0, "File exists but I don't have sufficient rights");
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_ABORT,
		 "File exists but I have no rights on it.");
  }

  // ok, now open the file
  DBG_DEBUG(0, "Opening file \"%s\"", name.c_str());
  f=fopen(name.c_str(), "rb");
  if (!f) {
    DBG_INFO(0,
	      "File exists but I have sufficient rights but can't open it");
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_ABORT,
		 "File exists, I have all rights but I could not open it.",
		 strerror(errno));
  }

  // read tag type and size
  buffer[0]=0;
  DBG_DEBUG(0, "Reading file \"%s\"", name.c_str());
  rv=fread(buffer, 3, 1, f);
  fclose(f);
  if (rv<0) {
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_ABORT,
		 "This seems not to be a key file.",
		 strerror(errno));
  }
  if (rv!=1) {
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_ABORT,
		 "This seems not to be a key file (bad read size).");
  }
  tmp=string(buffer, 3);
  DBG_DEBUG(0, "First three bytes are:");
  GWEN_Text_LogString(buffer, 3, 0, GWEN_LoggerLevelDebug);
  tlvType=String::typeTLV(tmp);
  if (tlvType!=MEDIUMKEYFILE_TAG_CRYPT &&
      tlvType!=MEDIUMKEYFILE_TAG_CRYPT_OLD) {
    DBG_INFO(0, "Bad tag type (%02x)", tlvType);
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_BAD_MEDIUM,
		 ERROR_ADVISE_ABORT,
		 "This seems not to be a key file (bad type).");
  }
  size=String::sizeTLV(tmp);
  if (size % 8) {
    DBG_INFO(0, "Bad tag size (%d)", size);
    return Error("KeyfilePlugin::mediumCheck",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_BAD_MEDIUM,
		 ERROR_ADVISE_ABORT,
		 "This seems not to be a key file (bad size).");
  }
  DBG_DEBUG(0, "File \"%s\" supported", name.c_str());
  return Error();
}





Pointer<Medium> KeyFilePlugin::createNewMedium(bool readonly,
					       int country,
					       const string &bankId,
					       const string &userId,
					       const string &name){
  Pointer<Medium> medium;
  Error err;

  // name empty ?
  if (name.empty())
    throw Error("KeyFilePlugin::createNewMedium()",
		ERROR_LEVEL_NORMAL,
		0,
		ERROR_ADVISE_DONTKNOW,
		"need name"
		"for RDH mode the name contains the path for the keyfile");

  // check if file already exists
  if (access(name.c_str(), F_OK)) {
    // File does not exist

    // create medium; Create new MediumRDH object
    MediumKeyfile *mediumtmp = new MediumKeyfile(hbci(),name);
    // Store it in the upcast version
    medium = mediumtmp;
    // For convenience, use a reference in the following
    MediumKeyfile &mediumrdh = *mediumtmp;

    string pin;

    err=hbci()->authentificator().ref().getSecret(0,
						 name,
						 pin,
						 true);
    if (!err.isOk())
      throw Error("KeyFilePlugin::createNewMedium",err);

    // create context
    err=mediumrdh.createMedium(country,
			       bankId,
			       userId,
			       pin);
    if (!err.isOk())
      throw Error("KeyFilePlugin::createNewMedium()", err);

    // mount medium without customer
    err=mediumrdh.mountMedium(pin);
    if (!err.isOk())
      throw Error("KeyFilePlugin::createNewMedium()", err);

    // select context
    err=mediumrdh.selectContext(country, bankId, userId);
    if (!err.isOk())
      throw Error("KeyFilePlugin::createNewMedium()", err);

    // create keys
    err=mediumrdh.createUserKeys();
    if (!err.isOk())
      throw Error("KeyFilePlugin::createNewMedium()", err);

    // unmount medium
    err=mediumrdh.unmountMedium(pin);
    if (!err.isOk())
      throw Error("KeyFilePlugin::createNewMedium()", err);
  } // if not exists
  else
    // File exists, so: create medium
    medium=new MediumKeyfile(hbci(),name);

  // finished

  return medium;
}


Pointer<Medium> KeyFilePlugin::mediumFromConfig(GWEN_DB_NODE *cfg) {
  string typ;
  string mname;
  Pointer<MediumKeyfile> m;
  int transform;

  if (!cfg) {
    throw Error("KeyFilePlugin::mediumFromConfig",
		ERROR_LEVEL_NORMAL,
		HBCI_ERROR_CODE_INVALID,
		ERROR_ADVISE_DONTKNOW,
		"no configuration given");
  }
  typ=GWEN_DB_GetCharValue(cfg, "mediumtype",0, "");
  if (strcasecmp(typ.c_str(), pluginName().c_str())!=0) {
    throw Error("KeyFilePlugin::mediumFromConfig",
		ERROR_LEVEL_NORMAL,
		HBCI_ERROR_CODE_INVALID,
		ERROR_ADVISE_DONTKNOW,
		"bad medium type");
  }
  mname=GWEN_DB_GetCharValue(cfg, "mediumname", 0,
			     GWEN_DB_GetCharValue(cfg, "keyfile",0, ""));
  if (mname.empty()) {
    throw Error("KeyFilePlugin::mediumFromConfig",
		ERROR_LEVEL_NORMAL,
		HBCI_ERROR_CODE_INVALID,
		ERROR_ADVISE_DONTKNOW,
		"missing medium name");
  }
  m=new MediumKeyfile(hbci(), mname);
  transform=GWEN_DB_GetIntValue(cfg, "transform",0, 0);
  if (transform) {
    DBG_NOTICE(0, "Transforming medium to new type");
    m.ref().transformToNewType();
  }
  return m.cast<Medium>();
}



Error KeyFilePlugin::mediumToConfig(Pointer<Medium> m,
				    GWEN_DB_NODE *cfg) {
  Pointer<MediumKeyfile> mkf;

  try {
    mkf=m.cast<MediumKeyfile>();
  }
  catch (Error xerr) {
    return Error("KeyFilePlugin::mediumToConfig",xerr);
  }
  GWEN_DB_SetCharValue(cfg,
                       GWEN_DB_FLAGS_DEFAULT |
                       GWEN_DB_FLAGS_OVERWRITE_VARS,
		       "mediumtype",
		       m.ref().mediumTypeName().c_str());
  GWEN_DB_SetCharValue(cfg,
                       GWEN_DB_FLAGS_DEFAULT |
                       GWEN_DB_FLAGS_OVERWRITE_VARS,
		       "mediumname",
		       mkf.ref().mediumName().c_str());
  return Error();
}




}; // namespace


HBCI::Error rdhfile_checkVersion() {
  int vmajor, vminor, vpl, vbuild;

  /* do some version checking
   * Please note that the constants used below are defined upon
   * compilation of this plugin. If you compile this plugin using
   * OpenHBCI 0.9.10 and try to use it on a system where OpenHBCI 0.9.9
   * is installed, this module will refuse to work. This plugin only
   * works with versions of OpenHBCI which are the same or newer than
   * the version used to compile this plugin.
   * If the major version differs then this module believes that the
   * version of OpenHBCI used is totally incompatible with the version
   * used to build this plugin.
   */
  HBCI::API::libraryVersion(vmajor, vminor, vpl, vbuild);
  if (vmajor!=OPENHBCI_VERSION_MAJOR) {
    /* this is the major compatibility check. The major version is
     * incremented if a newer version is incompatible with older ones */
    fprintf(stderr,
	    " Different major versions, "
	    "please recompile RDHFile plugin.\n");
    return HBCI::Error("Keyfile Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "Major version does not match");
  }
  if (!(
	((vminor==OPENHBCI_VERSION_MINOR)&&(vpl>OPENHBCI_VERSION_PATCHLEVEL))
	||
	((vminor==OPENHBCI_VERSION_MINOR) &&
	 (vpl==OPENHBCI_VERSION_PATCHLEVEL) &&
	 (vbuild>=OPENHBCI_VERSION_BUILD))
       )
     ) {
    fprintf(stderr,
	    "This plugin needs OpenHBCI v%d.%d.%d.%d or newer.\n",
	    OPENHBCI_VERSION_MAJOR,
	    OPENHBCI_VERSION_MINOR,
	    OPENHBCI_VERSION_PATCHLEVEL,
	    OPENHBCI_VERSION_BUILD);
    return HBCI::Error("RDHFile Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "need OpenHBCI v"OPENHBCI_VERSION_FULL_STRING
		       " or newer");
  }
  return HBCI::Error();
}


extern "C" {
  // The version of openhbci's plugin interface system (i.e. the
  // convention that this plugin has to define a registerYourSelf()
  // function.)
  int rdhfile_openhbci_plugin_interface_version = OPENHBCI_PLUGIN_INTERFACE_VERSION;

  // Version numbers of this plugin
  int rdhfile_openhbci_module_current=KEYFILE_MODULE_CURRENT;
  int rdhfile_openhbci_module_revision=KEYFILE_MODULE_REVISION;
  int rdhfile_openhbci_module_age=KEYFILE_MODULE_AGE;


  // The function that registers this plugin at the HBCI::API. This
  // function will be looked up by dlsym() and called from the plugin
  // interface system.
  HBCI::Error rdhfile_registerPlugin(HBCI::Hbci *hbci,
				     HBCI::PluginManager *mgr,
				     PluginType pt) {
    HBCI::Pointer<HBCI::Plugin> mp;
    HBCI::Error err;

    if (pt!=PluginTypeMedium) {
      return HBCI::Error("RDHFile Plugin",
			 ERROR_LEVEL_NORMAL,
			 HBCI_ERROR_CODE_MEDIUM,
			 ERROR_ADVISE_DONTKNOW,
			 "bad plugin type");
    }
    err=rdhfile_checkVersion();
    if (!err.isOk())
      return HBCI::Error("Keyfile Plugin",err);

    mp=new HBCI::KeyFilePlugin(hbci);
    return mgr->registerPlugin(mp);
  }

  /*
   * This function simply creates a medium plugin without registering it
   */
  HBCI::Pointer<HBCI::Plugin> rdhfile_createPlugin(HBCI::Hbci *hbci,
						   HBCI::PluginManager *mgr){
    HBCI::Pointer<HBCI::Plugin> mp;
    HBCI::Error err;

    err=rdhfile_checkVersion();
    if (!err.isOk())
      throw HBCI::Error("Keyfile Plugin",err);

    mp=new HBCI::KeyFilePlugin(hbci);
    mp.setDescription("KeyFilePlugin");
    return mp;
  }


}; // extern "C"
